searchNodes=[{"doc":"Methods for interacting with ArangoDB that aren't through Ecto.This allows for easy interaction with graph functions of Arango. Using Ecto's relations for edge relations was tried but it was found to be too much of a 'hacky' solution. Using separate functions that still utilise Ecto document queries was found to be the optimal solution.","ref":"ArangoXEcto.html","title":"ArangoXEcto","type":"module"},{"doc":"Runs a raw AQL query on the database.This will create a transaction and cursor on Arango and run the raw query.If there is an error in the query such as a syntax error, an Arangox.Error will be raised.Parametersrepo - The Ecto repo module to use for queriesquery - The AQL query string to executevars - A keyword list or a map with the values for variables in the queryopts - Options to be passed to DBConnection.transaction/3Examplesiex&gt; ArangoXEcto.aql_query( Repo, &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot;, fname: &quot;John&quot;, lname: &quot;Smith&quot; ) {:ok, [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot;, &quot;_key&quot; =&gt; &quot;12345&quot;, &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot;, &quot;first_name&quot; =&gt; &quot;John&quot;, &quot;last_name&quot; =&gt; &quot;Smith&quot; } ]}","ref":"ArangoXEcto.html#aql_query/4","title":"ArangoXEcto.aql_query/4","type":"function"},{"doc":"Checks if a collection existsThis will return true if the collection exists in the database, matches the specified type and is not a system database, otherwise it will be false.Parametersrepo - The Ecto repo module to use for the querycollection_name - Name of the collection to checktype - The type of collection to check against, defaults to a regular documentExamplesChecking a document collection existsiex&gt; ArangoXEcto.collection_exists?(Repo, :users) trueChecking an edge collection existsiex&gt; ArangoXEcto.collection_exists?(Repo, &quot;my_edge&quot;, :edge) trueChecking a system document collection exists does not workiex&gt; ArangoXEcto.collection_exists?(Repo, &quot;_system_test&quot;) false","ref":"ArangoXEcto.html#collection_exists?/3","title":"ArangoXEcto.collection_exists?/3","type":"function"},{"doc":"Creates an edge between two modulesThis can create an edge collection dynamically if no additional fields are required, otherwise an edge schema needs to be specified.The collection name can be passed as an option or is obtained from the provided schema, otherwise it is generated dynamically.Parametersrepo - The Ecto repo module to use for queriesfrom - The Ecto Schema struct to use for the from vertexto - The Ecto Schema struct to use for the to vertexopts - Options to useOptionsAccepts the following options::edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides collection_name.:fields - The values of the fields to set on the edge. Requires edge to be set otherwise it is ignored.:collection_name - The name of the collection to use.Examplesiex&gt; ArangoXEcto.create_edge(Repo, user1, user2) %ArangoXEcto.Edge{_from: &quot;users/12345&quot;, _to: &quot;users/54321&quot;}Create an edge with a specific edge collection nameiex&gt; ArangoXEcto.create_edge(Repo, user1, user2, collection_name: &quot;friends&quot;) %ArangoXEcto.Edge{_from: &quot;users/12345&quot;, _to: &quot;users/54321&quot;}Create a edge schema and use it to create an edge relationdefmodule UserPosts do use ArangoXEcto.Edge import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields() field(:type, :string) end def changeset(edge, attrs) do edges_changeset(edge, attrs) |&gt; cast(attrs, [:type]) |&gt; validate_required([:type]) end end iex&gt; ArangoXEcto.create_edge(Repo, user1, user2, edge: UserPosts, fields: %{type: &quot;wrote&quot;}) %ArangoXEcto.Edge{_from: &quot;users/12345&quot;, _to: &quot;users/54321&quot;}","ref":"ArangoXEcto.html#create_edge/4","title":"ArangoXEcto.create_edge/4","type":"function"},{"doc":"Same as create_edge/4 but specific for custom edge and fields.","ref":"ArangoXEcto.html#create_edge/6","title":"ArangoXEcto.create_edge/6","type":"function"},{"doc":"Deletes all edges matching matching the queryIf the :conditions option is set then those conditions must be true to delete.To just delete one edge do so like any other Ecto Schema struct, i.e. using Ecto.Repo methods.Parametersrepo - The Ecto repo module to use for queriesfrom - The Ecto Schema struct to use for the from vertexto - The Ecto Schema struct to use for the to vertexopts - Options to useOptionsAccepts the following options::edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides :collection_name.:collection_name - The name of the collection to use.:conditions - A keyword list of conditions to filter for edge deletionExamplesDeletes all edges from user1 to user2iex&gt; ArangoXEcto.delete_all_edges(Repo, user1, user2) :okDeletes all edges from user1 to user2 in specific collectioniex&gt; ArangoXEcto.delete_all_edges(Repo, user1, user2, collection_name: &quot;friends&quot;) :okDeletes all edges from user1 to user2 that have matching conditionsiex&gt; ArangoXEcto.delete_all_edges(Repo, user1, user2, conditions: [type: &quot;best_friend&quot;]) :ok","ref":"ArangoXEcto.html#delete_all_edges/4","title":"ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"Generates a edge schema dynamicallyIf no collection name is passed in the options, then one is generated using the passed modules.This will create the Ecto Module in the environment dynamically. It will create it under the closest common parent module of the passed modules plus the Edges alias. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.UsersPosts. This assumes that the edge collection name was generated and not passed in, if it was UsersPosts would be replaced with the camelcase of that collection name.Returns the Edge Module name as an atom.Parametersfrom_module - Ecto Schema Module for the from part of the edgeto_module - Ecto Schema Module for the to part of the edgeopts - Options passed for module generationOptions:collection_name - The name of collection to use instead of generating itExamplesiex&gt; ArangoXEcto.edge_module(MyProject.User, MyProject.Company, [collection_name: &quot;works_for&quot;]) MyProject.WorksFor iex&gt; ArangoXEcto.edge_module(MyProject.User, MyProject.Company) MyProject.UsersCompanies","ref":"ArangoXEcto.html#edge_module/3","title":"ArangoXEcto.edge_module/3","type":"function"},{"doc":"Gets an ID from a module and a keyParametersmodule - Module to get the collection name fromkey - The _key to use in the idExampleiex&gt; ArangoXEcto.get_id_from_module(User, &quot;123456&quot;) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_module/2","title":"ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"Gets an ID from a schema structParametersstruct - The Ecto structExampleIf the User schema's collection name is users the following would be:iex&gt; user = %User{id: &quot;123456&quot;} %User{id: &quot;123456&quot;} iex&gt; ArangoXEcto.get_id_from_struct(user) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_struct/1","title":"ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"Returns if a Schema is a document schema or notChecks for the presence of the __schema__/1 function on the module and not an edge.","ref":"ArangoXEcto.html#is_document?/1","title":"ArangoXEcto.is_document?/1","type":"function"},{"doc":"Returns if a Schema is an edge or notChecks for the presence of the __edge__/0 function on the module.","ref":"ArangoXEcto.html#is_edge?/1","title":"ArangoXEcto.is_edge?/1","type":"function"},{"doc":"Converts raw output of a query into a structTransforms string map arguments into atom key map, adds id key and drops _id, _key and _rev keys. Then it creates a struct from filtered arguments using module.If a list of maps are passed then the maps are enumerated over.Parametersmaps - List of maps or singular map to convert to a structmodule - Module to use for the structExampleiex&gt; {:ok, users} = ArangoXEcto.aql_query( Repo, &quot;FOR user IN users RETURN user&quot; ) {:ok, [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot;, &quot;_key&quot; =&gt; &quot;12345&quot;, &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot;, &quot;first_name&quot; =&gt; &quot;John&quot;, &quot;last_name&quot; =&gt; &quot;Smith&quot; } ]} iex&gt; ArangoXEcto.raw_to_struct(users, User) [ %User{ id: &quot;12345&quot;, first_name: &quot;John&quot;, last_name: &quot;Smith&quot; } ]","ref":"ArangoXEcto.html#raw_to_struct/2","title":"ArangoXEcto.raw_to_struct/2","type":"function"},{"doc":"Returns the type of a moduleThis is just a shortcut to using is_edge/1 and is_document/1. If it is neither nil is returned.ExamplesA real edge schemaiex&gt; ArangoXEcto.schema_type(MyApp.RealEdge) :edgeSome module that is not an Ecto schemaiex&gt; ArangoXEcto.schema_type(MyApp.RandomModule) nil","ref":"ArangoXEcto.html#schema_type/1","title":"ArangoXEcto.schema_type/1","type":"function"},{"doc":"Same as schema_type/1 but throws an error on noneThis is just a shortcut to using is_edge/1 and is_document/1. If it is neither an error is raised.ExamplesA real edge schemaiex&gt; ArangoXEcto.schema_type!(MyApp.RealEdge) :edgeSome module that is not an Ecto schemaiex&gt; ArangoXEcto.schema_type!(MyApp.RandomModule) ** (ArgumentError) Not an Ecto Schema","ref":"ArangoXEcto.html#schema_type!/1","title":"ArangoXEcto.schema_type!/1","type":"function"},{"doc":"","ref":"ArangoXEcto.html#t:mod/0","title":"ArangoXEcto.mod/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:query/0","title":"ArangoXEcto.query/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:vars/0","title":"ArangoXEcto.vars/0","type":"type"},{"doc":"Ecto adapter for ArangoDB using ArangoXThis implements methods for Ecto.Adapter. These functions should not be accessed directly and should only be used by Ecto. Direct interaction functions are in the ArangoXEcto module.","ref":"ArangoXEcto.Adapter.html","title":"ArangoXEcto.Adapter","type":"module"},{"doc":"Callback implementation for Ecto.Adapter.Schema.autogenerate/1.","ref":"ArangoXEcto.Adapter.html#autogenerate/1","title":"ArangoXEcto.Adapter.autogenerate/1","type":"function"},{"doc":"Checks out a connection for the duration of the given function.","ref":"ArangoXEcto.Adapter.html#checkout/3","title":"ArangoXEcto.Adapter.checkout/3","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Schema.delete/4.","ref":"ArangoXEcto.Adapter.html#delete/4","title":"ArangoXEcto.Adapter.delete/4","type":"function"},{"doc":"Returns the dumpers for a given type.","ref":"ArangoXEcto.Adapter.html#dumpers/2","title":"ArangoXEcto.Adapter.dumpers/2","type":"function"},{"doc":"Ensure all applications necessary to run the adapter are started","ref":"ArangoXEcto.Adapter.html#ensure_all_started/2","title":"ArangoXEcto.Adapter.ensure_all_started/2","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Queryable.execute/5.","ref":"ArangoXEcto.Adapter.html#execute/5","title":"ArangoXEcto.Adapter.execute/5","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Transaction.in_transaction?/1.","ref":"ArangoXEcto.Adapter.html#in_transaction?/1","title":"ArangoXEcto.Adapter.in_transaction?/1","type":"function"},{"doc":"Initialise adapter with config","ref":"ArangoXEcto.Adapter.html#init/1","title":"ArangoXEcto.Adapter.init/1","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Schema.insert/6.","ref":"ArangoXEcto.Adapter.html#insert/6","title":"ArangoXEcto.Adapter.insert/6","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Schema.insert_all/7.","ref":"ArangoXEcto.Adapter.html#insert_all/7","title":"ArangoXEcto.Adapter.insert_all/7","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_decimal/1","title":"ArangoXEcto.Adapter.load_decimal/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_float/1","title":"ArangoXEcto.Adapter.load_float/1","type":"function"},{"doc":"Returns the loaders of a given type.","ref":"ArangoXEcto.Adapter.html#loaders/2","title":"ArangoXEcto.Adapter.loaders/2","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Queryable.prepare/2.","ref":"ArangoXEcto.Adapter.html#prepare/2","title":"ArangoXEcto.Adapter.prepare/2","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Transaction.rollback/2.","ref":"ArangoXEcto.Adapter.html#rollback/2","title":"ArangoXEcto.Adapter.rollback/2","type":"function"},{"doc":"Starts the Agent with an empty list","ref":"ArangoXEcto.Adapter.html#start_link/1","title":"ArangoXEcto.Adapter.start_link/1","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Storage.storage_down/1.","ref":"ArangoXEcto.Adapter.html#storage_down/1","title":"ArangoXEcto.Adapter.storage_down/1","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Storage.storage_status/1.","ref":"ArangoXEcto.Adapter.html#storage_status/1","title":"ArangoXEcto.Adapter.storage_status/1","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Storage.storage_up/1.","ref":"ArangoXEcto.Adapter.html#storage_up/1","title":"ArangoXEcto.Adapter.storage_up/1","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Queryable.stream/5.","ref":"ArangoXEcto.Adapter.html#stream/5","title":"ArangoXEcto.Adapter.stream/5","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Transaction.transaction/3.","ref":"ArangoXEcto.Adapter.html#transaction/3","title":"ArangoXEcto.Adapter.transaction/3","type":"function"},{"doc":"Callback implementation for Ecto.Adapter.Schema.update/6.","ref":"ArangoXEcto.Adapter.html#update/6","title":"ArangoXEcto.Adapter.update/6","type":"function"},{"doc":"Defines an Arango Edge collection as an Ecto Schema Module.Edge modules are dynamically created in the environment if they don't already exist. This will define the required fields of an edge (_from and _to) and will define the default changeset.Collections in ArangoDB are automatically created if they don't exist already.Extending Edge SchemasIf you need to add additional fields to an edge, you can do so by creating your own edge module and defining the required fields as well as any additional fields. Luckily there are some helper macros so you don't have to do this manually again.A custom schema module must use this module by adding use ArangoXEcto.Edge.When defining the fields in your schema, make sure to call edge_fields/1. This will add the _from and _to fields to the schema. It does not have to be before any custom fields but it good convention to do so.A changeset/2 function is automatically defined on the custom schema module but this must be overridden this so that you can cast and validate the custom fields. The edges_changeset/2 method should be called to automatically implement the casting and validation of the _from and _to fields. It does not have to be before any custom field operations but it good convention to do so.Exampledefmodule MyProject.UserPosts do use ArangoXEcto.Edge import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields() field(:type, :string) end def changeset(edge, attrs) do edges_changeset(edge, attrs) |&gt; cast(attrs, [:type]) |&gt; validate_required([:type]) end end","ref":"ArangoXEcto.Edge.html","title":"ArangoXEcto.Edge","type":"behaviour"},{"doc":"","ref":"ArangoXEcto.Edge.html#c:changeset/2","title":"ArangoXEcto.Edge.changeset/2","type":"callback"},{"doc":"Macro to define the required edge fields i.e. _from and _to.This is required when using a custom edge schema and can be used as below.schema &quot;user_posts&quot; do edge_fields() field(:type, :string) end","ref":"ArangoXEcto.Edge.html#edge_fields/0","title":"ArangoXEcto.Edge.edge_fields/0","type":"macro"},{"doc":"Default changeset for an edge.Casts and requires the _from and _to fields. This will also verify the format of both fields to match that of an Arango id.Any custom changeset should first use this changeset.Direct use of the edges_changeset/2 function is discouraged unless per the use case mentioned above.ExampleTo add a required type field, you could do the following:def changeset(edge, attrs) do edges_changeset(edge, attrs) |&gt; cast(attrs, [:type]) |&gt; validate_required([:type]) end","ref":"ArangoXEcto.Edge.html#edges_changeset/2","title":"ArangoXEcto.Edge.edges_changeset/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Edge.html#t:t/0","title":"ArangoXEcto.Edge.t/0","type":"type"},{"doc":"Defines Ecto Migrations for ArangoDBNOTE: ArangoXEcto dynamically creates collections for you and this method is discouraged unless you need to define indexes.Migrations must use this module, otherwise migrations will not work. To do this, replace use Ecto.Migration with use ArangoXEcto.Migration.Since ArangoDB is schemaless, no fields need to be provided, only the collection name. First create a collection struct using the collection/2 function. Then pass the collection struct to the create/1 function. To create indexes it is a similar process using the index/3 function. Collections must be created BEFORE indexes.To drop the collection on a migration down, do the same as creation except use the drop/1 function instead of the create/1 function. Indexes are automatically removed when the collection is removed and cannot be deleted using the drop/1 function.Exampledefmodule MyProject.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create(collection(:users)) create(index(&quot;users&quot;, [:email])) end def down do drop(collection(:users)) end end","ref":"ArangoXEcto.Migration.html","title":"ArangoXEcto.Migration","type":"module"},{"doc":"Creates a collection structUsed to in functions that perform actions on the database.Accepts a collection type parameter that can either be :document or :edge, otherwise it will raise an error. The default option is :document.Examplesiex&gt; collection(&quot;users&quot;) %Collection{name: &quot;users&quot;, 2) iex&gt; collection(&quot;users&quot;, :edge) %Collection{name: &quot;users&quot;, 3)","ref":"ArangoXEcto.Migration.html#collection/2","title":"ArangoXEcto.Migration.collection/2","type":"function"},{"doc":"Creates an objectWill create the passed object, either a collection or an index.ExamplesCreate a collectioniex&gt; create(collection(&quot;users&quot;)) :okCreate an indexiex&gt; create(index(&quot;users&quot;, [:email]) :ok","ref":"ArangoXEcto.Migration.html#create/1","title":"ArangoXEcto.Migration.create/1","type":"function"},{"doc":"Deletes an objectWill delete an object passed, can only be a collection, indexes cannot be deleted here.Exampleiex&gt; drop(collection(&quot;users&quot;)) :ok","ref":"ArangoXEcto.Migration.html#drop/1","title":"ArangoXEcto.Migration.drop/1","type":"function"},{"doc":"Creates an edge collection structSame as passing :edge as the second parameter to collection/2.","ref":"ArangoXEcto.Migration.html#edge/1","title":"ArangoXEcto.Migration.edge/1","type":"function"},{"doc":"Creates an index structDefault index type is a hash. To change this pass the :type option in options.OptionsOptions only apply to the creation of indexes and has no effect when using the drop/1 function.:type - The type of index to createAccepts: :fulltext, :geo, :hash, :persistent, :skiplist or :ttl:unique - If the index should be unique, defaults to false (hash, persistent &amp; skiplist only):sparse - If index should be spares, defaults to false (hash, persistent &amp; skiplist only):deduplication - If duplication of array values should be turned off, defaults to true (hash &amp; skiplist only):minLength - Minimum character length of words to index (fulltext only):geoJson - If a geo-spatial index on a location is constructed and geoJson is true, then the order within the array is longitude followed by latitude (geo only):expireAfter - Time in seconds after a document's creation it should count as expired (ttl only)ExamplesCreate index on email fieldiex&gt; index(&quot;users&quot;, [:email]) %Index{collection_name: &quot;users&quot;, fields: [:email]}Create dual index on email and ph_number fieldsiex&gt; index(&quot;users&quot;, [:email, :ph_number]) %Index{collection_name: &quot;users&quot;, fields: [:email, :ph_number]}Create unique email indexiex&gt; index(&quot;users&quot;, [:email], unique: true) %Index{collection_name: &quot;users&quot;, fields: [:email], [unique: true]}","ref":"ArangoXEcto.Migration.html#index/3","title":"ArangoXEcto.Migration.index/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.html#t:index_option/0","title":"ArangoXEcto.Migration.index_option/0","type":"type"},{"doc":"Converts Ecto.Query structs into AQL syntax.This module is copied from https://github.com/ArangoDB-Community/arangodb_ecto/blob/master/lib/arangodb_ecto/query.ex. All credit goes to mpoeter, the original author. Please go check out the original of this file.This is an updated version for Ecto V3","ref":"ArangoXEcto.Query.html","title":"ArangoXEcto.Query","type":"module"},{"doc":"Creates an AQL query to fetch all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#all/1","title":"ArangoXEcto.Query.all/1","type":"function"},{"doc":"Creates an AQL query to delete all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#delete_all/1","title":"ArangoXEcto.Query.delete_all/1","type":"function"},{"doc":"Creates an AQL query to update all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#update_all/1","title":"ArangoXEcto.Query.update_all/1","type":"function"},{"doc":"This module is a helper to automatically specify the primary key.The primary key is the Arango _key field but the _id field is also provided.Schema modules should use this module by add use ArangoXEcto.Schema to the module. The only exception to this is if the collection is an edge collection, in that case refer to ArangoXEcto.Edge.Exampledefmodule MyProject.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name, :string field :last_name, :string timestamps() end @doc false def changeset(app, attrs) do app |&gt; cast(attrs, [:first_name, :last_name]) |&gt; validate_required([:first_name, :last_name]) end end","ref":"ArangoXEcto.Schema.html","title":"ArangoXEcto.Schema","type":"module"},{"doc":"Helper methods","ref":"ArangoXEcto.Utils.html","title":"ArangoXEcto.Utils","type":"module"},{"doc":"Get system database connection","ref":"ArangoXEcto.Utils.html#get_system_db/1","title":"ArangoXEcto.Utils.get_system_db/1","type":"function"},{"doc":"Generates a migration.The repository must be set under :ecto_repos in the current app configuration or given via the -r option.Examplesmix ecto.gen.migration create_users mix ecto.gen.migration create_users -r Custom.RepoThe generated migration filename will be prefixed with the current timestamp in UTC which is used for versioning and ordering.By default, the migration will be generated to the priv/YOUR_REPO/migrations directory of the current application but it can be configured to be any subdirectory of priv by specifying the :priv key under the repository configuration.This generator will automatically open the generated file if you have ECTO_EDITOR set in your environment variable.Command line options-r, --repo - the repo to generate migration for","ref":"Mix.Tasks.Ecto.Gen.Migration.Arango.html","title":"Mix.Tasks.Ecto.Gen.Migration.Arango","type":"task"},{"doc":"Runs Migration/Rollback functions from migration modules","ref":"Mix.Tasks.Ecto.Migrate.Arango.html","title":"Mix.Tasks.Ecto.Migrate.Arango","type":"task"},{"doc":"","ref":"Mix.Tasks.Ecto.Migrate.Arango.html#down/0","title":"Mix.Tasks.Ecto.Migrate.Arango.down/0","type":"function"},{"doc":"Sets up all necessary collection in _systems db for migrations and creates database","ref":"Mix.Tasks.Ecto.Setup.Arango.html","title":"Mix.Tasks.Ecto.Setup.Arango","type":"task"},{"doc":"ArangoX Ecto","ref":"readme.html","title":"ArangoX Ecto","type":"extras"},{"doc":"About the ProjectBuilt WithGetting StartedPrerequisitesInstallationUsageRoadmapContributingLicenseContactAcknowledgements","ref":"readme.html#table-of-contents","title":"ArangoX Ecto - Table of Contents","type":"extras"},{"doc":"After playing around with different packages that implemented ArangoDB in Elixir, I found that there wasn't a package that suited my needs. I needed ArangoDB to work with Ecto seamlessly but there was no up-to-date adapter for Ecto available. ArangoX Ecto uses the power of ArangoX to communicate with ArangoDB and Ecto for the API in Elixir. Ecto is integrated with many other packages and can now be used with ArangoDB thanks to this package.Built WithArangox","ref":"readme.html#about-the-project","title":"ArangoX Ecto - About The Project","type":"extras"},{"doc":"To get the adapter integrated with your project, follow these simple steps.PrerequisitesElixir 1.10+ / Erlang OTP 23+InstallationAdd the following line to your mix dependencies to get started.{:arangox_ecto, &quot;~&gt; 0.6&quot;}","ref":"readme.html#getting-started","title":"ArangoX Ecto - Getting Started","type":"extras"},{"doc":"To connect to the database you need to specify the config values like so:config :my_app, MyApp.Repo, database: &quot;my_db&quot;, endpoints: &quot;http://1.2.3.4:8529&quot;Only database and endpoints are required but there are other available options can be found in the Arangox docs.The database should be setup using$ mix ecto.setup.arangoBasic UsageRepo SetupTo use the adapter in your repo, make sure your repo uses the ArangoXEcto.Adapter module for the adapter.defmodule MyApp.Repo do use Ecto.Repo, otp_app: :my_app, adapter: ArangoXEcto.Adapter endSchema SetupSince ArangoDB uses a slightly different id system, your schema must use the ArangoXEcto.Schema instead of Ecto.Schema.defmodule MyApp.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name, :string field :last_name, :string timestamps() end @doc false def changeset(app, attrs) do app |&gt; cast(attrs, [:first_name, :last_name]) |&gt; validate_required([:first_name, :last_name]) end endMigration SetupThe adapter will automatically create collections if they don't already exist but there are cases where you might need to use migrations. For example, if you needed to create indexes as well, the following would be used.defmodule MyApp.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create(collection(:users)) create(index(&quot;users&quot;, [:email])) end def down do drop(collection(:users)) end endUsing migrations should be avoided unless necessary.Raw AQL queriesA lot of the time it is far more efficient to just run a raw AQL query, there's a function for that.ArangoXEcto.aql_query( Repo, &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot;, fname: &quot;John&quot;, lname: &quot;Smith&quot; )This query will return a result such as:{:ok, [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot;, &quot;_key&quot; =&gt; &quot;12345&quot;, &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot;, &quot;first_name&quot; =&gt; &quot;John&quot;, &quot;last_name&quot; =&gt; &quot;Smith&quot; } ]}This is awesome functionality, but a lot of the time we will want to resemble a specific struct. This is actually quite easy with the help of the ArangoXEcto.raw_to_struct/2 function. The same query above could be extended to also convert the output:ArangoXEcto.aql_query( Repo, &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot;, fname: &quot;John&quot;, lname: &quot;Smith&quot; ) |&gt; case do {:ok, results} -&gt; results |&gt; ArangoXEcto.raw_to_struct(User) {:error, _reason} -&gt; [] endThis will return something like:[ %User{ id: &quot;12345&quot;, first_name: &quot;John&quot;, last_name: &quot;Smith&quot; } ]This is clearly a much better representation of the result and can be used in further Ecto methods.Graph RelationsAfter a lot of tinkering, the best solution to graph relations in with Arango and Ecto was to not use Ecto a lot.The adapter will dynamically create and manage edge collections. Each edge collection will be created as an Ecto schema when they are first used. This will allow for more extensibility through ecto onto the edges. The module will be created under the closest common parent module of the passed modules plus the Edges alias. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.UsersPosts. This assumes that the edge collection name was generated and not explicitly defined, if it was UsersPosts would be replaced with the camel case of that collection name.To read more about Edge Schemas and how to extend edge schemas to add additional fields, read the docs on ArangoXEcto.Edge.To create and delete edges (as well as other useful methods) check out the full documentation.In order to delete a specific edge, you can do it exactly as you would any other ecto struct (since after all it is one).Querying of edges can be done either through using an AQL query or by using Ecto methods.Further UsageFor more examples and full documentation, please refer to the Documentation.","ref":"readme.html#usage","title":"ArangoX Ecto - Usage","type":"extras"},{"doc":"See the open issues for a list of proposed features (and known issues).Some planned ideas:Named Graph integrationsEasier Graph level functionsMulti-tenancy","ref":"readme.html#roadmap","title":"ArangoX Ecto - Roadmap","type":"extras"},{"doc":"Contributions are what make the open source community such an amazing place to be learn, inspire, and create. Any contributions you make are greatly appreciated.Fork the ProjectCreate your Feature Branch (git checkout -b feature/AmazingFeature)Write some awesome codeCommit your Changes (git commit -m 'Add some AmazingFeature')Push to the Branch (git push origin feature/AmazingFeature)Open a Pull Request","ref":"readme.html#contributing","title":"ArangoX Ecto - Contributing","type":"extras"},{"doc":"Distributed under the Apache 2.0 License. See LICENSE for more information.","ref":"readme.html#license","title":"ArangoX Ecto - License","type":"extras"},{"doc":"Tom Grozev - @tomgrozev - enquire@tomgrozev.comProject Link: https://github.com/TomGrozev/arangox_ecto","ref":"readme.html#contact","title":"ArangoX Ecto - Contact","type":"extras"},{"doc":"mpoeter - Wrote the original Ecto Query to AQL code","ref":"readme.html#acknowledgements","title":"ArangoX Ecto - Acknowledgements","type":"extras"}]